/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2015 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.extension;

import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.model.constraints.IConstraint;
import org.fortiss.tooling.kernel.model.constraints.IConstraintVerificationStatus;
import org.fortiss.tooling.kernel.service.IConstraintVerificationService;
import org.fortiss.tooling.kernel.service.IConstraintVerificationService.IFix;
import org.fortiss.tooling.kernel.service.base.IEObjectAware;

/**
 * Interface for constraint verifiers used by {@link IConstraintVerificationService}.
 * 
 * @author aravantinos
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 6CB960DB7263BB104C485D049A370ADE
 */
public interface IConstraintVerifier<T extends IConstraint> extends IEObjectAware<T> {
	/** Verify the given constraint. */
	void verify(T constraint);

	/**
	 * @param obj
	 *            Alters <code>obj</code> so that the contained items which should not be considered
	 *            when computing the checksum have been removed.
	 */
	void removeConstraintIrrelevantContent(EObject obj);

	/**
	 * Returns a list of possible automatic fixes for the (constraint of the) given status.
	 * A status should be candidate to fixing if it is anything but successful or outdated.
	 */
	List<IFix> fixes(IConstraintVerificationStatus status);

	/**
	 * The following features deal with the usage of constraints to drive the process.
	 * 
	 * EXPERIMENTAL.
	 */

	/** Gets a general description for the constraint type. */
	String getDescription();

	/** True if this constraint shall be displayed as a development process constraint. */
	boolean isUsableForDevelopmentProcess();
}
